/* Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.myfaces.portlet.faces.application;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.portlet.filter.RenderResponseWrapper;
import javax.portlet.RenderResponse;


public class PortletViewHandlerRenderResponseWrapper
  extends RenderResponseWrapper
  {
    private static final int WRITE_BUFFER_SIZE = 1024;

    private int mBufferSize = WRITE_BUFFER_SIZE;
    private ByteArrayOutputStream mBytes;
    private boolean mIsWritingBytes;
    private PrintWriter mPrintWriter;
    private boolean mIsWritingChars;
    
    public PortletViewHandlerRenderResponseWrapper(RenderResponse response)
    {
      super(response);
    }
    
    public PrintWriter getWriter() throws java.io.IOException
    {
      if (mBytes == null)
      {
          mBytes = new ByteArrayOutputStream(mBufferSize);
      }
      if (mPrintWriter == null)
      {
          mPrintWriter = new PrintWriter(new OutputStreamWriter(mBytes, this.getCharacterEncoding()),true);
      }
      
      mIsWritingChars = true;
      mIsWritingBytes = false;
      
      return mPrintWriter;
    }
    
    public OutputStream getPortletOutputStream() throws IOException
    {
        if (mBytes == null)
        {
            mBytes = new ByteArrayOutputStream(mBufferSize);
        }
        
      mIsWritingChars = false;
      mIsWritingBytes = true;
        
        return mBytes;
    }
    
    public void resetBuffer()
    {

        if (mPrintWriter != null) 
        {
            mPrintWriter.close();
            mPrintWriter = null;
        }
        
        reset();
            
        mIsWritingBytes = false;
        mIsWritingChars = false;
    }
    
    public void reset()
    {
      if (mBytes != null)
          mBytes.reset();
    }
    
    
    public void flushBuffer() throws java.io.IOException
    {
      if (mBytes == null) return; // nothing to do!
      
      
      // Move the data from the buffer to the response
      
      // First determine whether response has used a writer or outputstream
      try {
        copyToWriter(this.getResponse().getWriter());
      } 
      catch (IllegalStateException ise) {     
        mBytes.writeTo(this.getResponse().getPortletOutputStream());
      }
    }
    
    private void copyToWriter(PrintWriter writer) throws java.io.IOException
    {
      try {
        writer.write(mBytes.toString(this.getCharacterEncoding()));
      }
      catch (Exception e)
      {
        throw new IOException(e.toString());        
      }
    }
      
  
    
  }
